#ifndef __WAVE_FUN
#define __WAVE_FUN

#include "cwp.h"
#include "cstd.h"
#include "parser.h"

typedef struct {

  int nz;          /* number of gridpoints in z */
  int nx;          /* number of gridpoints in x */
  int nt;          /* number of time steps */
  int nm;          /* number of time steps to skip between movie frames
		      (<=0 for no movie) */
  float dz;        /* step in z */
  float dx;        /* step in x */
  float dt;        /* step in t */

  float freq;      /* center frequency of Ricker wavelet */
  
  int isz;         /* source depth, in units of dz */
  int isxbeg;      /* far left source x coord in units of dx */
  int isxend;      /* far right source x coord in units of dx */
  int iskip;       /* interval between sources in units of dx */
  int igz;         /* recvr depth, in units of dz */
  int igxbeg;      /* far left receiver x coord in units of dx */
  int igxend;      /* far right receiver x coord in units of dx */

  int imbeg;       /* midpoint begin */
  int imend;       /* midpoint end */
  int imskip;      /* midpoint skip */
  int ihmax;       /* offset radius, units of dx */

  char * vfile;    /* velocity file - defines space grid */
  char * tfile;    /* trace output file */
  char * mfile;    /* movie file */

  char * rmfile;   /* receiver movie file */
  char * imfile;   /* image file */

} WINFO;

/* read 
   
vfile
tfile
mfile
rmfile
imfile
nz
nx
dz
dx
nt
dt
nm
isz
isx
igz
igxbeg
igxend
     
from command line
*/
int getinputs(int argc, char ** argv, WINFO * wi);

/* assign constant to array - presumes memory already allocated */
void fassign(float * a, float c, int n);

/* assign zero to array - presumes memory already allocated */
void fzeros(float * a, int n);

/* square array elements - presumes memory already allocated
   and initialized */
void fsquare(float * a, int n);

/* get max val from array */
float fgetmax(float * a, int n);

/* get min val from array */
float fgetmin(float * a, int n);

/* return value at t of causal Ricker wavelet with peak frequency f */
float fgetrick(float t, float f);

#endif
