#include "wavefun.h"

int getinputs(int argc, char ** argv, WINFO * wi) {

  PARARRAY par;

  if (ps_createargs(&par,argc,argv)) {
    fprintf(stderr,"ERROR: getinputs failed to process command line\n");
    return 1;
  }

  wi->vfile=NULL;
  ps_ffcstring(par,"velocity",&(wi->vfile)); 
  /*
  if(ps_ffcstring(par,"trace",&(wi->tfile)) ||
     ps_ffcstring(par,"source",&(wi->mfile))){
     fprintf(stderr,"ERROR:no input trace or movie file\n");
     return 1;
  } 
  */
  wi->mfile=NULL;
  ps_ffcstring(par,"source",&(wi->mfile));
  wi->tfile=NULL;
  ps_ffcstring(par,"trace",&(wi->tfile));

  wi->rmfile=NULL;
  ps_ffcstring(par,"receiver",&(wi->rmfile));
  wi->imfile=NULL;
  ps_ffcstring(par,"image",&(wi->imfile));


  if (ps_ffint(par,"nz",&(wi->nz)) ||
      ps_ffint(par,"nx",&(wi->nx)) ||
      ps_ffint(par,"nt",&(wi->nt)) ||
      ps_ffreal(par,"dz",&(wi->dz))||
      ps_ffreal(par,"dx",&(wi->dx))||
      ps_ffreal(par,"dt",&(wi->dt))) {
    fprintf(stderr,"ERROR: getinputs\n");
    fprintf(stderr,"ERROR: wave requires all six of nx, nz, nt, dx, dz, dt\n");
    fprintf(stderr,"ERROR: to be input on the command line\n");
    return 1;
  }
  wi->nm=0;
  ps_ffint(par,"nm",&(wi->nm));
  wi->isz=1;
  ps_ffint(par,"isz",&(wi->isz));
  wi->isxbeg=(wi->nx)/2;
  ps_ffint(par,"isxbeg",&(wi->isxbeg));
  wi->isxend=(wi->nx)/2;
  ps_ffint(par,"isxend",&(wi->isxend));
  wi->iskip=1;
  ps_ffint(par,"iskip",&(wi->iskip));
  wi->igz=1;
  ps_ffint(par,"igz",&(wi->igz));
  wi->igxbeg=1;
  ps_ffint(par,"igxbeg",&(wi->igxbeg));
  wi->igxend=0;
  ps_ffint(par,"igxend",&(wi->igxend));
  wi->freq=0.01;
  ps_ffreal(par,"fpeak",&(wi->freq));
  
  /* default is zero offset */
  wi->ihmax=0;
  ps_ffint(par,"ihmax",&(wi->ihmax));
  wi->imbeg=wi->ihmax;
  ps_ffint(par,"imbeg",&(wi->imbeg));
  wi->imend=wi->nx-wi->ihmax-1;
  ps_ffint(par,"imend",&(wi->imend)); 
  wi->imskip=1;
  ps_ffint(par,"imskip",&(wi->imskip));

  /* sanity-check */
  if (wi->iskip<1 || wi->imskip<1) {
    fprintf(stderr,"ERROR: either source or midpoint skip is\n");
    fprintf(stderr,"nonpositive - ABORT\n");
    return 1;
  }
  if (wi->imbeg<wi->ihmax) {
    fprintf(stderr,"ERROR: first midpoint located within\n");
    fprintf(stderr,"offset radius of domain boundary - ABORT\n");
    return 1;
  }
  if (wi->imend>wi->nx-wi->ihmax-1) {
    fprintf(stderr,"ERROR: first midpoint located within\n");
    fprintf(stderr,"offset radius of domain boundary - ABORT\n");
    return 1;
  }
  if (wi->nx<1||wi->nz<1) {
    fprintf(stderr,"ERROR: number of spatial samples input is\n");
    fprintf(stderr,"nonpositive - ABORT\n");
    return 1;
  }
  if (wi->isxbeg<1 || wi->isxend>wi->nx-2 ||
      wi->isz<1 || wi->isz>wi->nz-2) {
    fprintf(stderr,"ERROR: source indices isz=%d, isxbeg=%d isxend=%d place source\n",
	    wi->isz,wi->isxbeg,wi->isxend);
    fprintf(stderr,"outside of wavefield update region [1,%d] x [1,%d]\n",
	    wi->nz-1,wi->nx-1);
    return 1;
  }

  if (wi->igz<1 || wi->igz>wi->nz-2 ||
      wi->igxbeg<1 || wi->igxbeg > wi->nx-2 ||
      wi->igxend<1 || wi->igxend > wi->nx-2) {
    fprintf(stderr,"ERROR: receiver depth index igz=%d or cable endpoint\n",
	    wi->igz);
    fprintf(stderr,"indices igxbeg=%d, igxend=%d place receivers outside of\n",
	    wi->igxbeg,wi->igxend);
    fprintf(stderr,"wavefield update region [1,%d] x [1,%d]\n",
	    wi->nz-1,wi->nx-1);
    return 1;
  }

  return 0;
}

void fassign(float * a, float c, int n) {
  int i;
  for (i=0;i<n;i++) a[i]=c;
}

void fzeros(float * a, int n) {
  int i;
  for (i=0;i<n;i++) a[i]=0.0;
}

void fsquare(float * a, int n) {
  int i;
  for (i=0;i<n;i++) a[i]=a[i]*a[i];
}

float fgetmax(float * a, int n) {
  int i;
  float m=-FLT_MAX;
  for (i=0;i<n;i++) m=fmaxf(m,a[i]);
  return m;
}

float fgetmin(float * a, int n) {
  int i;
  float m=FLT_MAX;
  for (i=0;i<n;i++) m=fminf(m,a[i]);
  return m;
}

float fgetrick(float t, float fpeak) {
  float st=PI*fpeak*(t-(1.2/fpeak));
  st*=st;
  return (1-2*st)*exp(-st);
}
